package com.loafle.overflow.module.generator.service;

import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.crawler.config.*;
import com.loafle.overflow.module.auth.model.AuthCrawler;
import com.loafle.overflow.module.auth.service.AuthCrawlerService;
import com.loafle.overflow.module.infra.model.Infra;
import com.loafle.overflow.module.infra.model.InfraService;
import com.loafle.overflow.module.meta.model.MetaSensorItemKey;
import com.loafle.overflow.module.meta.type.MetaCrawlerEnum;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("InfraServiceGenerator")
public class InfraServiceGenerator {

    @Autowired
    private GenerateUtil generateUtil;

    @Autowired
    private InfraServiceMysqlGenerator infraServiceMysqlGenerator;

    @Autowired
    private AuthCrawlerService authCrawlerService;

    public String process(Sensor dbSensor, List<SensorItem> sensorItems, Infra infra) throws IOException {
        com.loafle.overflow.module.infra.model.InfraService infraService = (com.loafle.overflow.module.infra.model.InfraService)infra;

        Config config = new Config();
        config.setId(String.valueOf(dbSensor.getId()));

        Target target = this.createTarget(infraService, dbSensor);

        if(target == null) {
            return null;
        }


        config.setTarget(target);

        // FIXME: Interval
        Schedule schedule = new Schedule();
        schedule.setInterval("5");
        config.setSchedule(schedule);

        Crawler crawler = this.generateUtil.getCrawler(dbSensor.getCrawler());
        config.setCrawler(crawler);

        Map<Integer, MetaSensorItemKey> keyMap = this.generateUtil.initMappingMap(dbSensor.getCrawler());

        if(dbSensor.getCrawler().getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
            this.infraServiceMysqlGenerator.process(sensorItems, keyMap, dbSensor, config);
        }

        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(config);
    }

    private Target createTarget(InfraService infraService, Sensor sensor) throws IOException {

        AuthCrawler authCrawler = this.authCrawlerService.readAuth(sensor.getCrawler(), sensor.getTarget());

        if(authCrawler == null) {
            return null;
        }

        Target target = new com.loafle.overflow.crawler.config.Target();
        Connection connection = new Connection();
        connection.setIp(StringConvertor.intToIp(infraService.getHost().getIp()));
        connection.setPort(String.valueOf(infraService.getPort()));
        connection.setPortType(infraService.getPortType());
        connection.setSsl(infraService.isTlsType());

        target.setConnection(connection);

        HashMap<String,String> optionMap;
        optionMap = new ObjectMapper().readValue(authCrawler.getAuthJson(), new TypeReference<HashMap<String,String>>() {});

        Map<String, Object> auth = new HashMap<>();

        if(sensor.getCrawler().getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
            auth.put("url", "jdbc:mysql://"+ StringConvertor.intToIp(infraService.getHost().getIp())+":"+String.valueOf(infraService.getPort()));
            auth.put("id", optionMap.get("ID")); // FIXME: Auth Info
            auth.put("pw", optionMap.get("PassWord")); // FIXME: Auth Info
        }

        target.setAuth(auth);

        return target;
    }

}
