package com.loafle.overflow.module.noauthprobe.service;

import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaNoAuthProbeStatus;
import com.loafle.overflow.module.meta.model.MetaProbeStatus;
import com.loafle.overflow.module.noauthprobe.dao.NoAuthProbeDAO;
import com.loafle.overflow.module.noauthprobe.model.NoAuthProbe;
import com.loafle.overflow.module.probe.model.Probe;
import com.loafle.overflow.module.probe.service.ProbeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("NoAuthProbeService")
public class NoAuthProbeService {

    @Autowired
    private NoAuthProbeDAO noAuthProbeDAO;

    @Autowired
    private ProbeService probeService;

    public NoAuthProbe regist(NoAuthProbe noAuthProbe) {

        noAuthProbe.setTempProbeKey(UUID.randomUUID().toString());
        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short)3));

        return this.noAuthProbeDAO.save(noAuthProbe);
    }


    public List<NoAuthProbe> readAllByDomain(Domain domain) {

        return this.noAuthProbeDAO.findAllByDomain(domain);
    }

    public NoAuthProbe read(long id) {
        return this.noAuthProbeDAO.findOne(id);
    }

    public List<NoAuthProbe> acceptNoAuthProbes(List<NoAuthProbe> noAuthProbes) {

        // Todo domain injection & member injection
        List<Probe> probes = new ArrayList<>();
        Probe probe = null;
        for (NoAuthProbe noAuth : noAuthProbes) {
            BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
            String encryptKey = passwordEncoder.encode(UUID.randomUUID().toString());

            probe = new Probe();
            probe.setEncryptionKey(encryptKey);
            probe.setProbeKey(noAuth.getApiKey());
            probe.setDomain(new Domain(1));
            probe.setAuthorizeMember(new Member(1));
            probe.setStatus(new MetaProbeStatus((short)1));
            probe.setTargetCount(0);
            probe.setSensorCount(0);

            String dispName = noAuth.getHostName().isEmpty() ?
                    StringConvertor.intToIp(noAuth.getIpAddress()) : noAuth.getHostName();
            probe.setDisplayName(dispName);

            probes.add(probe);
            noAuth.setStatus(new MetaNoAuthProbeStatus((short) 1));
        }

        this.probeService.regist(probes);
        this.noAuthProbeDAO.save(noAuthProbes);
        return this.readAllByDomain(noAuthProbes.get(0).getDomain());
    }

    public List<NoAuthProbe> denyNoauthProbes(List<NoAuthProbe> noAuthProbes) {
        for (NoAuthProbe noAuth : noAuthProbes) noAuth.setStatus(new MetaNoAuthProbeStatus((short) 2));
        this.noAuthProbeDAO.save(noAuthProbes);
        return this.readAllByDomain(noAuthProbes.get(0).getDomain());
    }
}
